package net.wanji.opt.cache;

import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.ControlModeEnum;
import net.wanji.common.enums.TurnConvertEnum;
import net.wanji.common.enums.WeekEnum;
import net.wanji.common.framework.Constants;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.opt.dto.*;
import net.wanji.opt.service.CrossSchedulesService;
import net.wanji.opt.service.CrossSchemeService;
import org.springframework.beans.BeanUtils;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.time.LocalTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/03/02 10:42
 */
@Component
@Slf4j
public class CrossDirTurnPhaseCache implements CommandLineRunner {

    private static Map<String, CrossPhaseDTO> phaseMap = new HashMap<>();

    @Resource
    CrossSchedulesService crossSchedulesService;
    @Resource
    CrossSchemeService crossSchemeService;

    /**
     * 当前时间路口运行的方案相位转向数据信息
     * 统一获取缓存方法
     * key: crossId-dir-turn value:CrossPhaseDTO
     *
     * @return
     */
    public Map<String, CrossPhaseDTO> getPhaseDirTurnMap() {
        return phaseMap;
    }

    /**
     * 使用前刷新当前缓存，当前缓存获取当前运行路口运行方案的相位车道信息
     */
    public void refresh() {
        init();
    }

    @Override
    public void run(String... args) {
        init();
    }

    /**
     * 初始化相位方向转向缓存
     *
     * key: crossId-dir-turn value:CrossPhaseDTO
     */
    private void init() {
        try {
            long start = System.currentTimeMillis();
            // 获取已执行的调度详情信息
            List<CrossSchedulesDTO> schedulesDTOList = crossSchedulesService.listCrossSchedulesDetails(Constants.SystemParam.NULL, Constants.SystemParam.YES);
            // 获取当前运行时段信息
            List<CrossSectionDTO> sectionInfos = listSection(schedulesDTOList);
            // 获取方案详情信息
            List<CrossSchemeDTO> schemeDTOList = crossSchemeService.listCrossSchemeDetails(Constants.SystemParam.NULL);
            Map<Integer, CrossSchemeDTO> schemeMap = schemeDTOList.stream().collect(Collectors.toMap(CrossSchemeDTO::getId, e -> e));

            String key;
            String crossId;
            Integer dir;
            String[] turnArr;
            Set<String> turnSet = new HashSet<>();
            for (CrossSectionDTO section : sectionInfos) {                       // 运行时段列表

                crossId = section.getCrossId();
                CrossSchemeDTO scheme = schemeMap.get(section.getSchemeId());   // 方案信息
                if (Objects.isNull(scheme.getPhaseInfos())) {
                    log.error("{}-路口， {}-方案，方案中相位信息不能为空", crossId, scheme.getId());
                    continue;
                }
                for (CrossPhaseDTO phase : scheme.getPhaseInfos()) {             // 相位信息列表

                    if (Objects.isNull(phase.getLightsInfos())) {
                        log.error("{}-路口， {}-方案，{}-灯组，灯组中车道信息不能为空", crossId, scheme.getId(), phase.getId());
                        continue;
                    }
                    for (CrossLightsDTO lights : phase.getLightsInfos()) {       // 灯组信息列表

                        dir = lights.getDir();
                        if (Objects.isNull(lights.getLaneInfos())) {
                            log.error("{}-路口， {}-方案，{}-灯组，灯组中车道信息不能为空", crossId, scheme.getId(), lights.getId());
                            continue;
                        }
                        for (LaneInfoDTO lane : lights.getLaneInfos()) {
                            // 车道转向转换为转向类型，并去重
                            turnArr = TurnConvertEnum.getCodeByKey(lane.getTurn()).split(Constants.SystemParam.SEPARATOR_UNDER_LINE);
                            turnSet.addAll(Arrays.stream(turnArr).collect(Collectors.toSet()));
                        }
                        for (String turn : turnSet) {
                            CrossPhaseDTO crossPhaseDTO = new CrossPhaseDTO();
                            BeanUtils.copyProperties(phase, crossPhaseDTO);
                            key = crossId + Constants.SystemParam.SEPARATOR_UNDER_LINE + dir + Constants.SystemParam.SEPARATOR_UNDER_LINE + turn;
                            crossPhaseDTO.setDirType(dir);
                            crossPhaseDTO.setTurnType(turn);
                            phaseMap.put(key, crossPhaseDTO);
                        }
                        turnSet.clear();
                    }
                }
            }
            long end = System.currentTimeMillis();
            log.info("加载路口当前运行的方案相位转向缓存耗时：{}ms,条数据,size={}", (end - start), phaseMap.size());
        } catch (Exception e) {
            log.error("相位方向转向缓存初始化失败", e);
        }
    }

    /**
     * 获取当前运行时段信息
     *
     * @param schedulesDTOList 调度列表
     * @return
     */
    private List<CrossSectionDTO> listSection(List<CrossSchedulesDTO> schedulesDTOList) {

        List<CrossSectionDTO> sectionInfos = new ArrayList<>();
        LocalTime currentTime = LocalTime.parse(DateUtil.getTime());
        int week = DateUtil.getWeek(new Date()) == Constants.SystemParam.ZERO ? WeekEnum.SUNDAY.getCode() : DateUtil.getWeek(new Date());   // 获取当前星期
        for (CrossSchedulesDTO schedules : schedulesDTOList) {

            if (Objects.equals(schedules.getWeek(), Constants.SystemParam.ZERO) &&
                    !DateUtil.getDate().equals(schedules.getSpecialDate())) {   // 先判断是否为特殊日期，如果特殊日期与当前日期不相同，则跳过
                continue;
            } else if (schedules.getWeek() != week) {
                continue;
            }

            // 时段信息列表
            if (Objects.nonNull(schedules.getSectionInfos())) {
                for (CrossSectionDTO section : schedules.getSectionInfos()) {
                    LocalTime startTime = LocalTime.parse(section.getStartTime());
                    LocalTime endTime = LocalTime.parse(section.getEndTime());
                    // 过滤非当前时段数据
                    if (currentTime.isBefore(startTime) || currentTime.isAfter(endTime)) {
                        continue;
                    }
                    // 过滤控制模式为“非定周期”的数据
                    if (!ControlModeEnum.FIXED_PERIOD.getCode().equals(section.getControlMode())) {
                        continue;
                    }
                    section.setPlanNo(schedules.getPlanNo());
                    sectionInfos.add(section);
                }
            }
        }
        return sectionInfos;
    }
}
